import random
from matplotlib.pyplot import plot, show, xlabel, ylabel, title

'''
Symulacja procesu losowego, gdzie punkt porusza sie w krokach po prostej rzeczywistej.
- Parametrami procesu sa (prawdopodobienstwo) p=prob oraz n > 0 naturalne.
- Punkt rozpoczyna wedrowke w pozycji 0.
- W kazdym kroku, punkt porusza sie o 1 w prawo z prawdopodobienstwem p, w lewo z prawd. 1-p.
- Proces konczy sie, gdy punkt osiagnie pozycje n lub -n.
"Sukces": dojscie do pozycji n.
"Porazka": dojscie do pozycji -n.
Szacujemy metoda Monte Carlo prawdopodobienstwo sukcesu dla zadanych p=prob i n.
'''


def random_dir(prob):
    if random.random() < prob:
        return 1
    return -1


def random_walk(prob, n):
    assert n != 0
    pos = 0
    while abs(pos) != n:
        pos += random_dir(prob)

    if pos > 0:
        return 1
    return 0


def success_rate(prob, n, tries=1000):
    assert tries != 0
    successes = 0
    for i in range(tries):
        if random_walk(prob, n) == 1:
            successes += 1
    return successes / tries


if __name__ == "__main__":
    print(success_rate(0.5, 10, 10000))

    xs = [i/100 for i in range(101)]
    ys = list(map(lambda x: success_rate(x, 10, 10000), xs))  # o "lambda" jeszcze nie bylo / "lambda" is left for a later lecture

    xlabel('Probability of moving right at each step')
    ylabel('Probability of ending at +10')
    title('Walking point simulation for n=10')
    plot(xs, ys)
    show()
